//=============================================================================
// RPG Maker MZ - Custom Game Day Status Interface
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Custom Game Day Status Interface
 * @author Evilcat
 *
 */

const day7 = ["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"];
const time3 = ["Morning","Afternoon","Evening"];
const dayF = ["st","nd","rd","th"];

(() => {

    Scene_Map.prototype.createAllWindows = function() {
    	this.createMapNameWindow();
	this.createDayStatusWindow();
    	Scene_Message.prototype.createAllWindows.call(this);
    	if ($gameSwitches.value(18)) {  // Check if switch 18 is ON
            if (!this._titleField2) { this.createTitleField2(); }
            this.createTitleParticles();
            this._titleField2.children.sort((a, b) => a.z - b.z); // Sort particles
    	}
    };

    Scene_Map.prototype.createDayStatusWindow = function() {
    	const rect = this.dayStatusWindowRect();
    	this._dayStatusWindow = new Window_DayStatus(rect);
    	this.addWindow(this._dayStatusWindow);
    };

    Scene_Map.prototype.dayStatusWindowRect = function() {
    	const wx = -16;
    	const wy = -12;
    	const ww = 420;
    	const wh = 96;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Map.prototype.update = function() {
    	Scene_Message.prototype.update.call(this);
    	this.updateDestination();
    	this.updateMenuButton();
    	this.updateMapNameWindow();
    	this.updateMainMultiply();
    	if (this.isSceneChangeOk()) {
            this.updateScene();
    	} else if (SceneManager.isNextScene(Scene_Battle)) {
            this.updateEncounterEffect();
    	}
	if(updateLove){
	    this._dayStatusWindow.refresh();
	    updateLove = false;
	}
	if($gameScreen._fadeOutDuration>0 && this._dayStatusWindow.contentsOpacity>0){
	    this._dayStatusWindow.contentsOpacity -= 16;
	}else if($gameScreen._fadeInDuration>0 && this._dayStatusWindow.contentsOpacity<255){
	    this._dayStatusWindow.contentsOpacity += 16;
	}
    	this.updateWaitCount();

    	if (this._tparticles && $gameSwitches.value(18)) {
            for (var i = 0; i < this._tparticles.length; i++) {
            	this._tparticles[i].update();
            }
    	}
    };

//-----------------------------------------------------------------------------
// Window_DayStatus
//
// The window for displaying the Day Status.

    function Window_DayStatus() {
    	this.initialize(...arguments);
    }

    Window_DayStatus.prototype = Object.create(Window_Base.prototype);
    Window_DayStatus.prototype.constructor = Window_DayStatus;

    Window_DayStatus.prototype.initialize = function(rect) {
    	Window_Base.prototype.initialize.call(this, rect);
    	if($gameMap._mapId==1 || $gameMap._mapId==10 || $gameMap._mapId==15)this.visible = false;
    	this.deactivate();
    	this.refresh();
    };

    Window_DayStatus.prototype.itemPadding = function() {
    	return 0;
    };

    Window_DayStatus.prototype.colSpacing = function() {
    	return 0;
    };

    Window_DayStatus.prototype.loadWindowskin = function() {
    	;
    };

    Window_DayStatus.prototype.drawBG = function() {
    	const bitmap = ImageManager.loadSystem("daystatusBG");
	bitmap.addLoadListener(() => {
	    this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 0, 0);
 	});
    };

    Window_DayStatus.prototype.resetFontSettings = function() {
    	this.contents.fontFace = $gameSystem.menuFontFace();
    	this.contents.fontSize = $gameSystem.mainFontSize();
    	this.resetTextColor();
    };

    Window_DayStatus.prototype.resetTextColor = function() {
    	this.changeTextColor(ColorManager.systemColor());
    	this.changeOutlineColor(ColorManager.menuOutlineColor());
	this.contents.outlineWidth = 5;
    };

    Window_DayStatus.prototype.refresh = function() {
    	this.contents.clear();
	let cday = $gameVariables.value(1);
	let dayS = cday-1;
	let daytext = day7[cday%7]+" "+cday+(dayS<3?dayF[dayS]:dayF[3])+" "+time3[$gameVariables.value(2)%3];
	const bitmap = ImageManager.loadSystem("daystatusBG");
	bitmap.addLoadListener(() => {
	    this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 0, 0);
	    this.drawText($gameVariables.value(3), -6, 28, 96, "center");
	    this.drawText(daytext, 96, 8, 300, "left");
 	});
	
    };

    Window_DayStatus.prototype.open = function() {
    	this.refresh();
    	Window_Base.prototype.open.call(this);
    };

})();