//=============================================================================
// RPG Maker MZ - Custom Game Menu Interface
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Custom Game Menu Interface
 * @author Evilcat
 *
 */

(() => {

    Sprite_Button.prototype.blockWidth = function() {
    	return 50;
    };

    Sprite_Button.prototype.blockHeight = function() {
    	return 72;
    };

    Sprite_Button.prototype.buttonData = function() {
    	const buttonTable = {
        	cancel: { x: 0, w: 2 },
        	pageup: { x: 2, w: 1 },
        	pagedown: { x: 3, w: 1 },
        	down: { x: 4, w: 1 },
        	up: { x: 5, w: 1 },
        	down2: { x: 6, w: 1 },
        	up2: { x: 7, w: 1 },
        	ok: { x: 8, w: 2 },
       		menu: { x: 10, w: 1 },
		mail: { x: 11, w: 2 },
		newmail: { x: 13, w: 2 },
		skip: { x: 15, w: 2 },
		item: { x: 17, w: 2 },
		save: { x: 19, w: 2 },
		settings: { x: 21, w: 2 },
		title: { x: 23, w: 2 }
    	};
    	return buttonTable[this._buttonType];
    };

    Sprite_Button.prototype.updateOpacity = function() {
    	this.opacity = 255;
    };

    Sprite_Button.prototype.updateFrame = function() {
    	const frame = this.isPressed() ? this._hotFrame : this._coldFrame;
	if(this.isPressed() && this._buttonType=="mail")$gameSwitches.setValue(3,false);
    	if (frame) {
            this.setFrame(frame.x, frame.y, frame.width, frame.height);
    	}
    };

    Scene_Map.prototype.createMenuButton = function() {

    	this._mailButton = new Sprite_Button("mail");
    	this._mailButton.x = Graphics.boxWidth - 604;
    	this._mailButton.y = this.buttonY();
	updateMail=true;
    	this._mailButton.visible = false;
    	this.addWindow(this._mailButton);

    	this._skipButton = new Sprite_Button("skip");
    	this._skipButton.x = Graphics.boxWidth - 504;
    	this._skipButton.y = this.buttonY();
    	this._skipButton.visible = false;
    	this.addWindow(this._skipButton);

    	this._itemButton = new Sprite_Button("item");
    	this._itemButton.x = Graphics.boxWidth - 404;
    	this._itemButton.y = this.buttonY();
    	this._itemButton.visible = false;
    	this.addWindow(this._itemButton);

    	this._saveButton = new Sprite_Button("save");
    	this._saveButton.x = Graphics.boxWidth - 304;
    	this._saveButton.y = this.buttonY();
    	this._saveButton.visible = false;
    	this.addWindow(this._saveButton);

    	this._settingsButton = new Sprite_Button("settings");
    	this._settingsButton.x = Graphics.boxWidth - 204;
    	this._settingsButton.y = this.buttonY();
    	this._settingsButton.visible = false;
    	this.addWindow(this._settingsButton);

    	this._titleButton = new Sprite_Button("title");
    	this._titleButton.x = Graphics.boxWidth - 104;
    	this._titleButton.y = this.buttonY();
    	this._titleButton.visible = false;
    	this.addWindow(this._titleButton);
    };

    Scene_Map.prototype.updateMenuButton = function() {
    	if(this._mailButton) {
            const menuEnabled = this.isMenuEnabled();
            if (menuEnabled === this._menuEnabled) {
		$gameSwitches.setValue(4, this._menuEnabled);
            	this._mailButton.visible = this._menuEnabled;
		this._skipButton.visible = this._menuEnabled;
		this._itemButton.visible = this._menuEnabled;
            	this._saveButton.visible = this._menuEnabled;
		this._settingsButton.visible = this._menuEnabled;
		this._titleButton.visible = this._menuEnabled;
            } else {
            	this._menuEnabled = menuEnabled;
            }
	    if(updateMail){
		if($gameSwitches.value(3))this._mailButton.setColdFrame(650, 0, 100, 72);
		else this._mailButton.setColdFrame(550, 0, 100, 72);
		updateMail = false;
	    }
    	}
    };

    Scene_Map.prototype.hideMenuButton = function() {
        this._mailButton.visible = false;
	this._skipButton.visible = false;
	this._itemButton.visible = false;
        this._saveButton.visible = false;
	this._settingsButton.visible = false;
	this._titleButton.visible = false;
        this._menuEnabled = false;
    };

    function canSaveSkip(){
	if($gameMap._mapId>2 && $gameMap._mapId<10)return true;
	else return false;
    }

    Scene_Map.prototype.updateCallMenu = function() {
    	if (this.isMenuEnabled() && !$gamePlayer.isMoving()) {
	    if (this.isMailCalled()) {
		this.menuCalling = false;
		this.callMail();
	    }else if (this.isMenuCalled() && canSaveSkip()) {
		this.menuCalling = true;
		this.callMenu();
	    }else if (this.isItemCalled()) {
		this.menuCalling = true;
		this.callItem();
	    }else if (this.isSaveCalled() && canSaveSkip()) {
		this.menuCalling = true;
		this.callSave();
	    }else if (this.isSettingsCalled()) {
		this.menuCalling = true;
		this.callSettings();
	    }else if (this.isTitleCalled()) {
		this.menuCalling = true;
		this.callTitle();
	    }
    	} else {
            this.menuCalling = false;
    	}
    };

    Scene_Map.prototype.isMailCalled = function() {
    	return Input.isTriggered("mail");
    };

    Scene_Map.prototype.isMenuCalled = function() {
    	return Input.isTriggered("skip") || TouchInput.isCancelled();
    };

    Scene_Map.prototype.isItemCalled = function() {
    	return Input.isTriggered("item");
    };

    Scene_Map.prototype.isSaveCalled = function() {
    	return Input.isTriggered("save");
    };

    Scene_Map.prototype.isSettingsCalled = function() {
    	return Input.isTriggered("settings");
    };

    Scene_Map.prototype.isTitleCalled = function() {
    	return Input.isTriggered("title");
    };

    Scene_Map.prototype.callMail = function() {
	AudioManager.playMe(characterVoice("dm_206"));
    	$gameTemp.clearDestination();
    	$gameTemp.reserveCommonEvent(2);
    };

    Scene_Map.prototype.callMenu = function() {
	AudioManager.playMe(characterVoice("dm_221"));
    	SceneManager.push(Scene_Skip);
    	Window_MenuCommand.initCommandPosition();
    	$gameTemp.clearDestination();
    	this._mapNameWindow.hide();
    	this._waitCount = 2;
    };

    Scene_Map.prototype.callItem = function() {
	AudioManager.playMe(characterVoice("dm_212"));
    	SceneManager.push(Scene_Item);
    	Window_MenuCommand.initCommandPosition();
    	$gameTemp.clearDestination();
    	this._mapNameWindow.hide();
    	this._waitCount = 2;
    };

    Scene_Map.prototype.callSave = function() {
	AudioManager.playSe(characterVoice("system_savegame"));
    	SceneManager.push(Scene_Save);
    	Window_MenuCommand.initCommandPosition();
    	$gameTemp.clearDestination();
    	this._mapNameWindow.hide();
    	this._waitCount = 2;
    };

    Scene_Map.prototype.callSettings = function() {
	AudioManager.playMe(characterVoice("dm_207"));
    	SceneManager.push(Scene_Options);
    	Window_MenuCommand.initCommandPosition();
    	$gameTemp.clearDestination();
    	this._mapNameWindow.hide();
    	this._waitCount = 2;
    };

    Scene_Map.prototype.callTitle = function() {
	AudioManager.playSe(characterVoice("system_endgame"));
    	SceneManager.push(Scene_GameEnd);
    	Window_MenuCommand.initCommandPosition();
    	$gameTemp.clearDestination();
    	this._mapNameWindow.hide();
    	this._waitCount = 2;
    };

    Scene_GameEnd.prototype.createBackground = function() {
    	Scene_MenuBase.prototype.createBackground.call(this);
    	this.setBackgroundOpacity(200);
    };

    Scene_GameEnd.prototype.commandWindowRect = function() {
    	const ww = 420;
    	const wh = this.calcWindowHeight(2, true);
    	const wx = (Graphics.boxWidth - ww) / 2;
    	const wy = (Graphics.boxHeight - wh) / 2;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Window_GameEnd.prototype.maxCols = function() {
        return 2;
    };

    Window_GameEnd.prototype.drawItem = function(index) {
    	const rect = this.itemLineRect(index);
    	const align = this.itemTextAlign();
    	this.resetTextColor();
    	this.changePaintOpacity(this.isCommandEnabled(index));
	this.drawText("タイトルに戻りますか？", 0, 0, 400, "center");
    	this.drawText(this.commandName(index), rect.x, rect.y, rect.width, align);
    };

    Window_GameEnd.prototype.itemRect = function(index) {
    	const maxCols = this.maxCols();
    	const itemWidth = this.itemWidth();
    	const itemHeight = this.itemHeight();
    	const colSpacing = this.colSpacing();
    	const rowSpacing = this.rowSpacing();
    	const col = index % maxCols;
    	const row = Math.floor(index / maxCols);
    	const x = col * itemWidth + colSpacing / 2 - this.scrollBaseX();
    	const y = row * itemHeight + rowSpacing / 2 - this.scrollBaseY();
    	const width = itemWidth - colSpacing;
    	const height = itemHeight - rowSpacing;
    	return new Rectangle(x, y+42, width, height);
    };

    Scene_Shop.prototype.helpWindowRect = function() {
    	const wx = 400;
    	const wy = this.helpAreaTop();
    	const ww = Graphics.boxWidth/2;
	const wh = this.calcWindowHeight(1, false); 	
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.commandWindowRect = function() {
    	const wx = 400;
    	const wy = this.mainAreaTop();
    	const ww = Graphics.boxWidth/2;
    	const wh = this.calcWindowHeight(1, true);
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.dummyWindowRect = function() {
    	const wx = 400;
    	const wy = this._commandWindow.y + this._commandWindow.height;
    	const ww = Graphics.boxWidth/2;
    	const wh = this.mainAreaHeight() - this._commandWindow.height;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.numberWindowRect = function() {
    	const wx = 400;
    	const wy = this._dummyWindow.y;
    	const ww = Graphics.boxWidth/2;
    	const wh = this._dummyWindow.height;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.goldWindowRect = function() {
    	const ww = this.mainCommandWidth();
    	const wh = this.calcWindowHeight(1, true);
    	const wx = Graphics.boxWidth - ww;
    	const wy = this.mainAreaTop();
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.buyWindowRect = function() {
    	const wx = 400;
    	const wy = this._dummyWindow.y;
    	const ww = Graphics.boxWidth/2;
    	const wh = this._dummyWindow.height;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.categoryWindowRect = function() {
    	const wx = 400;
    	const wy = this._dummyWindow.y;
    	const ww = Graphics.boxWidth/2;
    	const wh = this.calcWindowHeight(1, true);
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.sellWindowRect = function() {
    	const wx = 400;
    	const wy = this._categoryWindow.y + this._categoryWindow.height;
    	const ww = Graphics.boxWidth/2;
    	const wh = this.mainAreaHeight() - this._commandWindow.height - this._categoryWindow.height;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Shop.prototype.activateBuyWindow = function() {
    	this._buyWindow.setMoney(this.money());
    	this._buyWindow.show();
    	this._buyWindow.activate();
    };

    Scene_Shop.prototype.onSellOk = function() {
    	this._item = this._sellWindow.item();
    	this._categoryWindow.hide();
    	this._sellWindow.hide();
    	this._numberWindow.setup(this._item, this.maxSell(), this.sellingPrice());
    	this._numberWindow.setCurrencyUnit(this.currencyUnit());
    	this._numberWindow.show();
    	this._numberWindow.activate();
    };

    Window_ShopBuy.prototype.drawItem = function(index) {
    	const item = this.itemAt(index);
    	const price = this.price(item);
    	const rect = this.itemLineRect(index);
    	const priceWidth = this.priceWidth() + 50;
    	const priceX = rect.x + rect.width - priceWidth;
    	const nameWidth = rect.width - priceWidth;
    	this.changePaintOpacity(this.isEnabled(item));
    	this.drawItemName(item, rect.x, rect.y, nameWidth);
	this.drawText(item.itypeId==1?"プレゼント":"その他", priceX-150, rect.y, 100, "left");
	this.drawText($gameParty.numItems(item), priceX-50, rect.y, 50, "right");
    	this.drawText(price+"円", priceX, rect.y, priceWidth, "right");
    	this.changePaintOpacity(true);
    };

    Scene_Item.prototype.create = function() {
    	Scene_ItemBase.prototype.create.call(this);
	this.createHelpWindow();
    	this.createCategoryWindow();
    	this.createGoldWindow();
    	this.createItemWindow();
    	this.createActorWindow();
    };

    Scene_Item.prototype.createGoldWindow = function() {
    	const rect = this.goldWindowRect();
    	this._goldWindow = new Window_Gold(rect);
    	this.addWindow(this._goldWindow);
    };

    Scene_Item.prototype.goldWindowRect = function() {
    	const wx = 750;
    	const wy = this.mainAreaTop() + 100;
    	const ww = 220;
    	const wh = this.calcWindowHeight(1, true);
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Item.prototype.categoryWindowRect = function() {
    	const wx = 470;
    	const wy = this.mainAreaTop() + 100;
    	const ww = 280;
    	const wh = this.calcWindowHeight(1, true);
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Item.prototype.itemWindowRect = function() {
    	const wx = 470;
    	const wy = this._categoryWindow.y + this._categoryWindow.height;
    	const ww = 500;
    	const wh = 300;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Item.prototype.helpWindowRect = function() {
    	const wx = 470;
    	const wy = this.mainAreaTop() + 466;
    	const ww = 500;
	const wh = this.calcWindowHeight(1, false); 	
    	return new Rectangle(wx, wy, ww, wh);
    };

    Window_ItemCategory.prototype.maxCols = function() {
    	return 2;
    };

    Window_ItemList.prototype.maxCols = function() {
        return 5;
    };

    Window_ItemList.prototype.lineHeight = function() {
    	return 80;
    };

    Window_ItemList.prototype.colSpacing = function() {
    	return 8;
    };

    Window_ItemList.prototype.drawItem = function(index) {
    	const item = this.itemAt(index);
    	if (item) {
            const numberWidth = this.numberWidth();
            const rect = this.itemLineRect(index);
	    const iconY = rect.y + (this.lineHeight() - ImageManager.iconHeight) / 2;
            this.changePaintOpacity(this.isEnabled(item));
	    this.drawIcon(item.iconIndex, rect.x, iconY);
	    this.drawText($gameParty.numItems(item), rect.x+4, rect.y, rect.width, "right");
            this.changePaintOpacity(1);
    	}
    };

    Window_Help.prototype.itemPadding = function() {
    	return 0;
    };

//-----------------------------------------------------------------------------
// Scene_Skip
//
// The scene class of the time skip screen.

    function Scene_Skip() {
    	this.initialize(...arguments);
    }

    Scene_Skip.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Skip.prototype.constructor = Scene_Skip;

    Scene_Skip.prototype.initialize = function() {
    	Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_Skip.prototype.create = function() {
    	Scene_MenuBase.prototype.create.call(this);
    	this.createCommandWindow();
    };

    Scene_Skip.prototype.stop = function() {
    	Scene_MenuBase.prototype.stop.call(this);
    	this._commandWindow.close();
    };

    Scene_Skip.prototype.createBackground = function() {
    	Scene_MenuBase.prototype.createBackground.call(this);
    	this.setBackgroundOpacity(200);
    };

    Scene_Skip.prototype.createCommandWindow = function() {
    	const rect = this.commandWindowRect();
    	this._commandWindow = new Window_GameSkip(rect);
    	this._commandWindow.setHandler("skipTime", this.commandSkip.bind(this));
    	this._commandWindow.setHandler("cancel", this.popScene.bind(this));
    	this.addWindow(this._commandWindow);
    };

    Scene_Skip.prototype.commandWindowRect = function() {
    	const ww = 420;
    	const wh = this.calcWindowHeight(2, true);
    	const wx = (Graphics.boxWidth - ww) / 2;
    	const wy = (Graphics.boxHeight - wh) / 2;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Skip.prototype.commandSkip = function() {
    	$gameTemp.reserveCommonEvent(1);
	SceneManager.pop();
    };

//-----------------------------------------------------------------------------
// Window_GameSkip
//
// The window for selecting "Skip" on the menu.

    function Window_GameSkip() {
    	this.initialize(...arguments);
    }

    Window_GameSkip.prototype = Object.create(Window_Command.prototype);
    Window_GameSkip.prototype.constructor = Window_GameSkip;

    Window_GameSkip.prototype.initialize = function(rect) {
    	Window_Command.prototype.initialize.call(this, rect);
    	this.openness = 0;
    	this.open();
    };

    Window_GameSkip.prototype.makeCommandList = function() {
    	this.addCommand("Ok", "skipTime");
    	this.addCommand(TextManager.cancel, "cancel");
    };

    Window_GameSkip.prototype.maxCols = function() {
        return 2;
    };

    Window_GameSkip.prototype.drawItem = function(index) {
    	const rect = this.itemLineRect(index);
    	const align = this.itemTextAlign();
    	this.resetTextColor();
    	this.changePaintOpacity(this.isCommandEnabled(index));
	this.drawText("現在の時刻をスキップしますか？", 0, 0, 400, "center");
    	this.drawText(this.commandName(index), rect.x, rect.y, rect.width, align);
    };

    Window_GameSkip.prototype.itemRect = function(index) {
    	const maxCols = this.maxCols();
    	const itemWidth = this.itemWidth();
    	const itemHeight = this.itemHeight();
    	const colSpacing = this.colSpacing();
    	const rowSpacing = this.rowSpacing();
    	const col = index % maxCols;
    	const row = Math.floor(index / maxCols);
    	const x = col * itemWidth + colSpacing / 2 - this.scrollBaseX();
    	const y = row * itemHeight + rowSpacing / 2 - this.scrollBaseY();
    	const width = itemWidth - colSpacing;
    	const height = itemHeight - rowSpacing;
    	return new Rectangle(x, y+42, width, height);
    };

})();