//=============================================================================
// RPG Maker MZ - Custom Save & Settings Screen
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Custom save and settings interface.
 * @author Evilcat
 *
 */

ConfigManager.playerSpeed = 5;

(() => {

    DataManager.makeSavefileInfo = function() {
	let cday = $gameVariables.value(1);
    	const info = {};
    	info.title = $dataSystem.gameTitle;
        info.characters = $gameParty.charactersForSavefile();
        info.faces = $gameParty.facesForSavefile();
    	info.name = $gameActors.actor(1)._name;
	info.day7 = day7[cday%7];
    	info.day = cday+(cday-1<3?dayF[cday-1]:dayF[3]);
    	info.time = time3[$gameVariables.value(2)%3];
	info.love = $gameVariables.value(3);
	info.playtime = $gameSystem.playtimeText();
    	info.timestamp = Date();
    	return info;
    };

    Window_SavefileList.prototype.maxCols = function() {
        return 2;
    };

    Window_SavefileList.prototype.numVisibleRows = function() {
    	return 4;
    };

    Window_SavefileList.prototype.maxItems = function() {
    	return DataManager.maxSavefiles();
    };

    Window_SavefileList.prototype.indexToSavefileId = function(index) {
    	return index;
    };

    Window_SavefileList.prototype.savefileIdToIndex = function(savefileId) {
    	return savefileId;
    };

    Window_SavefileList.prototype.selectSavefile = function(savefileId) {
    	const index = Math.max(0, this.savefileIdToIndex(savefileId));
    	this.select(index);
    	this.setTopRow(index-4);
    };

    Window_SavefileList.prototype.drawTitle = function(savefileId, x, y) {
	const bitmap = ImageManager.loadSystem("pen");
	this.contents.blt(bitmap, 0, 0, 80, 80, x+16, y+8);
	this.contents.fontFace = $gameSystem.menuFontFace();
	this.changeTextColor(ColorManager.systemColor());
	this.changeOutlineColor(ColorManager.menuOutlineColor());
	this.contents.outlineWidth = 5;
	if (savefileId === 0) {
            this.drawText(TextManager.autosave, x+6, y+24, 150);
	} else {
            this.drawText(" " + TextManager.file + " " + (savefileId<10?"0"+savefileId:savefileId), x+8, y+24, 150);
	}

    };

    Window_SavefileList.prototype.drawContents = function(info, rect) {
	const x = rect.x + 140;
	const y = rect.y + 10;
	const heart = Math.floor(info.love/10);
	this.resetFontSettings();
	this.changeTextColor(ColorManager.systemColor());
	this.contents.outlineWidth = 3;
	this.drawText(info.name, x, y, 200);	
	for(let i=0;i<heart;i++)this.drawIcon(1072, x+216+(i*32), y);
	this.drawText(info.day7+" "+info.day+" "+info.time, x, y+45, 400);
	this.drawText(info.playtime, x+400, y+45, 130);
	this.resetTextColor();
	this.drawText(info.timestamp, rect.x, y+90, 680);
    };

    ConfigManager.makeData = function() {
    	const config = {};
    	config.alwaysDash = this.alwaysDash;
	config.playerSpeed = this.playerSpeed;
    	config.commandRemember = this.commandRemember;
    	config.touchUI = this.touchUI;
    	config.bgmVolume = this.bgmVolume;
    	config.bgsVolume = this.bgsVolume;
    	config.meVolume = this.meVolume;
    	config.seVolume = this.seVolume;
    	return config;
    };

    ConfigManager.applyData = function(config) {
    	this.alwaysDash = this.readFlag(config, "alwaysDash", false);
	this.playerSpeed = this.readSpeed(config, "playerSpeed");
    	this.commandRemember = this.readFlag(config, "commandRemember", false);
    	this.touchUI = this.readFlag(config, "touchUI", true);
    	this.bgmVolume = this.readVolume(config, "bgmVolume");
    	this.bgsVolume = this.readVolume(config, "bgsVolume");
    	this.meVolume = this.readVolume(config, "meVolume");
    	this.seVolume = this.readVolume(config, "seVolume");
    };

    ConfigManager.readSpeed = function(config, name) {
    	if (name in config) {
            return Number(config[name]).clamp(5, 7);
    	} else {
            return 5;
    	}
    };

    Scene_Options.prototype.optionsWindowRect = function() {
    	const n = Math.min(this.maxCommands(), this.maxVisibleCommands());
    	const ww = 400;
    	const wh = this.calcWindowHeight(7, true);
    	const wx = (Graphics.boxWidth - ww) / 2;
    	const wy = (Graphics.boxHeight - wh) / 2;
    	return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Options.prototype.maxCommands = function() {
    	return 5;
    };

    Window_Options.prototype.addGeneralOptions = function() {
    	this.addCommand("移動速度", "playerSpeed");
    };

    Window_Options.prototype.volumeOffset = function() {
    	return 10;
    };

    Window_Options.prototype.drawItem = function(index) {
    	const title = this.commandName(index);
    	const status = this.statusText(index);
    	const rect = this.itemLineRect(index);
    	const statusWidth = this.statusWidth();
    	const titleWidth = rect.width - statusWidth;
    	this.resetTextColor();
    	this.changePaintOpacity(this.isCommandEnabled(index));
	this.drawText("Player Settings", 0, 0, 400, "center");
	this.drawText("Audio Settings", 0, 88, 400, "center");
    	this.drawText(title, rect.x, rect.y, titleWidth, "left");
    	this.drawText(status, rect.x + titleWidth, rect.y, statusWidth, "right");
    };

    Window_Options.prototype.itemRect = function(index) {
    	const maxCols = this.maxCols();
    	const itemWidth = this.itemWidth();
    	const itemHeight = this.itemHeight();
    	const colSpacing = this.colSpacing();
    	const rowSpacing = this.rowSpacing();
    	const col = index % maxCols;
    	const row = Math.floor(index / maxCols);
    	const x = col * itemWidth + colSpacing / 2 - this.scrollBaseX();
    	const y = row * itemHeight + rowSpacing / 2 - this.scrollBaseY();
    	const width = itemWidth - colSpacing;
    	const height = itemHeight - rowSpacing;
	if(index==0)return new Rectangle(x, y+42, width, height);
	else return new Rectangle(x, y+86, width, height);
    };

    Window_Options.prototype.statusText = function(index) {
    	const symbol = this.commandSymbol(index);
    	const value = this.getConfigValue(symbol);
    	if (this.isVolumeSymbol(symbol)) {
            return this.volumeStatusText(value);
    	} else {
	    if(value==5)return "普通";
       	    else if(value==6)return "速い";
	    else return "超高速";
    	}
    };

    Window_Options.prototype.changeSpeed = function(forward) {
    	let lastValue = this.getConfigValue("playerSpeed");
    	if(forward)lastValue++;
	else lastValue--;
    	if (lastValue > 7) {
            this.changeValue("playerSpeed", 5);
    	}else if (lastValue < 5) {
            this.changeValue("playerSpeed", 7);
    	}else{
            this.changeValue("playerSpeed", lastValue);
	}
    };

    Window_Options.prototype.processOk = function() {
    	const index = this.index();
    	const symbol = this.commandSymbol(index);
    	if (this.isVolumeSymbol(symbol)) {
            this.changeVolume(symbol, true, true);
    	} else {
            this.changeSpeed(true);
    	}
    };

    Window_Options.prototype.cursorRight = function() {
    	const index = this.index();
    	const symbol = this.commandSymbol(index);
    	if (this.isVolumeSymbol(symbol)) {
            this.changeVolume(symbol, true, false);
    	} else {
            this.changeSpeed(true);
    	}
    };

    Window_Options.prototype.cursorLeft = function() {
    	const index = this.index();
    	const symbol = this.commandSymbol(index);
    	if (this.isVolumeSymbol(symbol)) {
            this.changeVolume(symbol, false, false);
    	} else {
            this.changeSpeed(false);
    	}
    };

    Window_Options.prototype.setConfigValue = function(symbol, volume) {
    	ConfigManager[symbol] = volume;
	$gamePlayer._moveSpeed = ConfigManager.playerSpeed;
    };

})();
