//=============================================================================
// RPG Maker MZ - Custom Title Screen
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Custom tile screen layout.
 * @author Evilcat
 */

(() => {

    Scene_Splash.prototype.adjustBackground = function() {
    	this.centerSprite(this._backSprite);
    };

    Scene_Splash.prototype.initWaitCount = function() {
    	if (this.isEnabled()) {
	    AudioManager.playSe(circleVoice("EvilcatoVoice"));
            this._waitCount = 60;
    	} else {
            this._waitCount = 0;
    	}
    };

    Scene_Title.prototype.create = function() {
        Scene_Base.prototype.create.call(this);
        this.createBackground();
        this.createForeground();
    	this.createWindowLayer();
    	this.createCommandWindow();
	this._waitCount = 200;
    };

    Scene_Title.prototype.playTitleMusic = function() {
	AudioManager.playBgm($dataSystem.titleBgm);
    	AudioManager.playBgs(createRainBgs("Rain2"));
    	AudioManager.stopMe();
    };

    Scene_Title.prototype.createBackground = function() {
    	this._backSprite1 = new Sprite(
            ImageManager.loadTitle1($dataSystem.title1Name)
    	);
    	this._backSprite2 = new Sprite(
            ImageManager.loadTitle1("eyes")
    	);
    	this.addChild(this._backSprite1);
    	this.addChild(this._backSprite2);
    };

    Scene_Title.prototype.adjustBackground = function() {
    	this.centerSprite(this._backSprite1);
	this._backSprite2.x = 1450;
        this._backSprite2.y = 236;
    };

    Scene_Title.prototype.drawGameTitle = function() {
    	const x = 15;
    	const y = Graphics.height - 45;
    	const maxWidth = Graphics.width - x * 2;
    	const text = $dataSystem.gameTitle;
    	const bitmap = this._gameTitleSprite.bitmap;
    	bitmap.fontFace = $gameSystem.mainFontFace();
    	bitmap.outlineColor = "black";
    	bitmap.outlineWidth = 3;
    	bitmap.fontSize = 20;
    	bitmap.drawText(text, x, y, maxWidth, 48, "left");
    };

    Scene_Title.prototype.commandWindowRect = function() {
    	const offsetX = $dataSystem.titleCommandWindow.offsetX;
    	const offsetY = $dataSystem.titleCommandWindow.offsetY;
    	const ww = this.mainCommandWidth();
    	const wh = this.calcWindowHeight(4, true);
    	const wx = (Graphics.boxWidth - ww) / 2 + offsetX;
    	const wy = Graphics.boxHeight - wh - 96 + offsetY;
    	return new Rectangle(90, 270, 380, 400);
    };

    Scene_Title.prototype.commandNewGame = function() {
	AudioManager.playSe(characterVoice("system_gamestart"));
    	DataManager.setupNewGame();
    	this._commandWindow.close();
    	this.fadeOutAll();
    	SceneManager.goto(Scene_Map);
    };

    Scene_Title.prototype.commandContinue = function() {
	AudioManager.playSe(characterVoice("system_loadgame"));
    	this._commandWindow.close();
    	SceneManager.push(Scene_Load);
    };

    Scene_Title.prototype.commandOptions = function() {
	AudioManager.playMe(characterVoice("dm_207"));
    	this._commandWindow.close();
    	SceneManager.push(Scene_Options);
    };

    Scene_Title.prototype.commandQuit = function() {
        SceneManager.exit();
    };

    Scene_Title.prototype.createCommandWindow = function() {
    	const background = $dataSystem.titleCommandWindow.background;
    	const rect = this.commandWindowRect();
    	this._commandWindow = new Window_TitleCommand(rect);
    	this._commandWindow.setBackgroundType(background);
    	this._commandWindow.setHandler("newGame", this.commandNewGame.bind(this));
    	this._commandWindow.setHandler("continue", this.commandContinue.bind(this));
    	this._commandWindow.setHandler("options", this.commandOptions.bind(this));
	this._commandWindow.setHandler("quit", this.commandQuit.bind(this));
	this._commandWindow.opacity = 0;
    	this.addWindow(this._commandWindow);
    };

    Scene_Title.prototype.update = function() {
	if(this._waitCount--<=0){
	    if(this._backSprite2.x==1450){
		this._waitCount=20;
		this._backSprite2.x = 716;
	    }else{
		this._waitCount=200;
		this._backSprite2.x = 1450;
	    }
	}
    	if (!this.isBusy()) {
            this._commandWindow.open();
    	}
    	Scene_Base.prototype.update.call(this);
    };

    Window_TitleCommand.prototype.playOkSound = function() {
    	;
    };

    Window_TitleCommand.prototype.itemRect = function(index) {
    	const maxCols = this.maxCols();
    	const itemWidth = this.itemWidth();
    	const itemHeight = this.itemHeight();
    	const colSpacing = this.colSpacing();
    	const rowSpacing = this.rowSpacing();
    	const col = index % maxCols;
    	const row = Math.floor(index / maxCols);
    	const x = col * itemWidth + colSpacing / 2 - this.scrollBaseX();
    	const y = row * itemHeight + rowSpacing / 2 - this.scrollBaseY();
    	const width = itemWidth - colSpacing;
    	const height = itemHeight - rowSpacing;
	return new Rectangle(x, row*72, 380, 72);
    };

    Window_TitleCommand.prototype.drawBackgroundRect = function(rect) {
    	const c1 = "rgba(0, 0, 0, 0)";
    	const c2 = "rgba(0, 0, 0, 0)";
    	const x = rect.x;
    	const y = rect.y;
    	const w = rect.width;
    	const h = rect.height;
    };

    Window_TitleCommand.prototype.makeCommandList = function() {
    	const continueEnabled = this.isContinueEnabled();
        this.addCommand(" ", 'newGame');
        this.addCommand(" ", 'continue', this.isContinueEnabled());
        this.addCommand(" ", 'options');
	this.addCommand(" ", 'quit', false);
    };

    Window_NameEdit.prototype.setup = function(actor, maxLength) {
    	this._actor = actor;
    	this._maxLength = maxLength;
    	this._name = actor.name().slice(0, this._maxLength);
    	this._index = this._name.length;
    	this._defaultName = this._name;
    };

    Window_NameEdit.prototype.left = function() {
    	const nameCenter = (this.innerWidth) / 2;
    	const nameWidth = (this._maxLength + 1) * this.charWidth();
    	return Math.min(nameCenter - nameWidth / 2, this.innerWidth - nameWidth);
    };

    Window_NameEdit.prototype.itemRect = function(index) {
    	const x = this.left() + index * this.charWidth();
    	const y = 80;
    	const width = this.charWidth();
    	const height = this.lineHeight();
    	return new Rectangle(x, y, width, height);
    };

    Window_NameEdit.prototype.refresh = function() {
        this.contents.clear();
	if(this._actor._actorId==1)this.drawText("お名前を入力してください。", 100, 30);
	else this.drawText("ヒロインを何とお呼びしたいですか？", 35, 30);
        for (let i = 0; i < this._maxLength; i++) {
            this.drawUnderline(i);
    	}
    	for (let j = 0; j < this._name.length; j++) {
            this.drawChar(j);
    	}
    	const rect = this.itemRect(this._index);
    	this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
    };

    Window_NameInput.prototype.table = function() {
        return [
            Window_NameInput.JAPAN1,
            Window_NameInput.JAPAN2,
            Window_NameInput.JAPAN3
        ];
    };

    Window_NameEdit.prototype.charWidth = function() {
    	const text = "\uff21";
    	return this.textWidth(text);
    };

})();
