//=============================================================================
// RPG Maker MZ - Custom Walking Sprite
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Change walking sprite to sequence 6.
 * @author Evilcat
 */

(() => {

    const _Game_CharacterBase_initMembers = Game_CharacterBase.prototype.initMembers;
    Game_CharacterBase.prototype.initMembers = function() {
	_Game_CharacterBase_initMembers.call(this);
    	this._ecchiCount = 0;
    };

    Game_CharacterBase.prototype.maxPattern = function() {
    	return 6;
    };

    Game_CharacterBase.prototype.pattern = function() {
    	return this._pattern < 6 ? this._pattern : 0;
    };

    Game_CharacterBase.prototype.setPattern = function(pattern) {
    	this._pattern = pattern;
	this._animationCount = 0;
    };

    Game_CharacterBase.prototype.animationWait = function() {
	//idle time sprite
	if(this._moveFrequency==1){
	    return this._pattern == 5 ? 8 : 40;
	}else if(this._moveFrequency==2){
	    return 20;
	}else if(this._moveFrequency==3){
	    return 8;
	}else if(this._moveFrequency==4){
	    return 6;
	}else{
	    return 2;
	}
    };

    Game_CharacterBase.prototype.setCum = function() {
	this._pattern = 1;
	this._animationCount = 0;
	this._direction = 8;
	this._moveFrequency = 3;
    };


    Game_CharacterBase.prototype.update = function() {
    	if (this.isStopping()) {
            this.updateStop();
    	}
    	if (this.isJumping()) {
            this.updateJump();
    	} else if (this.isMoving()) {
            this.updateMove();
    	}
	if (this._moveFrequency==4 || this._moveFrequency==5) {
	    this._ecchiCount++;
	    if(this._direction==4 && this._ecchiCount==200){
		this._direction=6;
		this._ecchiCount=0;
	    }else if(this._direction==6 && this._ecchiCount==8){
		this._direction=4;
		this._ecchiCount=0;
	    }
	}
    	this.updateAnimation();
    };

    Game_CharacterBase.prototype.straighten = function() {
    	if (this.hasWalkAnime() || this.hasStepAnime()) {
            this._pattern = 0;
    	}
    	this._animationCount = 0;
    };

    Game_CharacterBase.prototype.resetPattern = function() {
    	this.setPattern(0);
    };

    Game_CharacterBase.prototype.jumpfast = function(xPlus, yPlus) {
    	this._x += xPlus;
    	this._y += yPlus;
	this._realX = this._x;
        this._realY = this._y;
    	this.resetStopCount();
    	this.straighten();
    };

    Game_Player.prototype.initMembers = function() {
    	Game_Character.prototype.initMembers.call(this);
    	this._vehicleType = "walk";
    	this._vehicleGettingOn = false;
    	this._vehicleGettingOff = false;
    	this._dashing = false;
    	this._needsMapReload = false;
    	this._transferring = false;
    	this._newMapId = 0;
    	this._newX = 0;
    	this._newY = 0;
    	this._newDirection = 0;
    	this._fadeType = 0;
    	this._followers = new Game_Followers();
    	this._encounterCount = 0;
	this._moveSpeed = ConfigManager.playerSpeed;
    };

    Game_Player.prototype.refresh = function() {
    	const actor = $gameParty.leader();
    	let characterName = actor ? actor.characterName() : "";
    	const characterIndex = actor ? actor.characterIndex() : 0;
	//check if it need umbrella
	if(needUmbrella())characterName = "$player_um";
    	this.setImage(characterName, characterIndex);
    	this._followers.refresh();
    };

    function needUmbrella(){
    	if($gameMap._mapId==2){
	    return true;
	}else if($gameSwitches.value(2) && $gameParty.hasItem($dataItems[2],false)){
	    if($gameMap._mapId>=6 && $gameMap._mapId<=9)return true;
	}
	return false;
    }

    Game_Player.prototype.jump = function(xPlus, yPlus) {
    	this._x += xPlus;
    	this._y += yPlus;
    	const distance = Math.round(Math.sqrt(xPlus * xPlus + yPlus * yPlus));
    	this._jumpPeak = 10 + distance - this._moveSpeed;
    	this._jumpCount = this._jumpPeak * 2;
    	this.resetStopCount();
    	this.straighten();
    };

    Game_Player.prototype.isDashing = function() {
    	return false;
    };

    Game_Player.prototype.animationWait = function() {
    	return (9 - this.realMoveSpeed()) * 3;
    };

    Game_Player.prototype.updateNonmoving = function(wasMoving, sceneActive) {
    	if (!$gameMap.isEventRunning()) {
           if (wasMoving) {
            	$gameParty.onPlayerWalk();
            	this.checkEventTriggerHere([1, 2]);
            	if ($gameMap.setupStartingEvent()) {
                    return;
            	}
           }else{
		if(this._direction!=8)this.setDirection(8);
	   }
           if (sceneActive && this.triggerAction()) {
            	return;
           }
           if (wasMoving) {
            	this.updateEncounterCount();
           } else {
            	$gameTemp.clearDestination();
           }
    	}
    };
	
    Game_Player.prototype.triggerTouchAction = function() {
    	if ($gameTemp.isDestinationValid()) {
            const direction = this.direction();
            const x1 = this.x;
            const y1 = this.y;
            const x2 = $gameMap.roundXWithDirection(x1, direction);
            const y2 = $gameMap.roundYWithDirection(y1, direction);
            const x3 = $gameMap.roundXWithDirection(x2, direction);
            const y3 = $gameMap.roundYWithDirection(y2, direction);
            const destX = $gameTemp.destinationX();
            let destY = $gameTemp.destinationY();
	    if(destY==5)destY=6;
	    if(this._direction==2)this.setDirection(8);
            if (x1 === destX && destY === 6) {
	    	if(this._direction!=8)this.setDirection(8);
            	return this.triggerTouchActionD4(x1, y1);
            }
    	}
    	return false;
    };

    Game_Player.prototype.triggerTouchActionD4 = function() {
    	this.checkEventTriggerThere([0, 1, 2]);
    	return $gameMap.setupStartingEvent();
    };

    Game_Player.prototype.checkEventTriggerThere = function(triggers) {
    	if (this.canStartLocalEvents()) {
            const direction = this.direction();
            const x1 = this.x;
            const y1 = this.y;
            this.startMapEvent(x1, 6, triggers, true);
    	}
    };

    Game_Temp.prototype.isDestinationValid = function() {
    	return this._destinationX !== null && this._destinationY>1;
    };

    Sprite_Character.prototype.patternWidth = function() {
    	if (this._tileId > 0) {
            return $gameMap.tileWidth();
    	} else if (this._isBigCharacter) {
            return this.bitmap.width / 6;
    	} else {
            return this.bitmap.width / 12;
    	}
    };

    Sprite_Destination.prototype.initialize = function() {
    	Sprite.prototype.initialize.call(this);
    	this.createBitmap();
    	this._frameCount = 0;
	this.visible = false;
    };

    Sprite_Destination.prototype.update = function() {
    	Sprite.prototype.update.call(this);
    };

})();